package meerkat;

/**
 * Created by Vladimir Eliezer Tokarev on 1/22/2016.
 * provides voters management options
 */
public interface RegistryInstance {

    /**
     * Adds new voter to the bulletin-board
     *
     * @param voterID      id of the new user
     * @param personalData for example residence location
     * @param callBack     when the adding voter done callBack.HandleVoterAdded will be called
     * @return void
     */
    abstract void AddVoter(String voterID, String personalData, RegistryCallBack callBack);

    /**
     * Adding given voter to given group
     *
     * @param voterID  the voter id
     * @param groupID  the group id to which we want add the voter
     * @param callBack when the adding voter done callBack.HandleVoterAddedToGroup will be called
     * @return true if the adding action succeeded else return false
     */
    abstract void AddToGroup(String voterID, String groupID, RegistryCallBack callBack);

    /**
     * Removes given voter from given group
     *
     * @param voterID  the voter id
     * @param groupID  the group from which we want to remove the user
     * @param callBack when the adding voter done callBack.HandleVoterRemovedFromGroup will be called
     * @return true if the removing action succeeded else return false
     */
    abstract void RemoveFromGroup(String voterID, String groupID, RegistryCallBack callBack);

    /**
     * Sets that the voter have voted
     *
     * @param id       id tag string
     * @param callBack when the adding voter done callBack.HandleSetVoted will be called
     * @return true if the set voted succeed else false
     */
    abstract void SetVoted(String id, RegistryCallBack callBack);

    /**
     * Requests all the groups that the given id voter is in
     *
     * @param id       id tag string
     * @param callBack when the adding voter done callBack.HandleVoterGroups will be called
     * @return list of groups ids (or names), if the method fails its empty
     */
    abstract void GetGroups(String id, RegistryCallBack callBack);

    /**
     * Retrieves list of strings that represents voter
     *
     * @param id       id tag string
     * @param callBack when the adding voter done callBack.HandleVoterInfo will be called
     * @return list of strings (empty list if the lookup failed)
     */
    abstract void GetPersonIDDetails(String id, RegistryCallBack callBack);
}

